#!/bin/sh -e

# Skript pro automatickou instalaci OMR700 Recorderu (firmware)
# Priklada se do aktualizacniho balicku. Pri upgrade je spusten
# a provede instalaci nove verze firmware
# Zakladni spousteni: s parametrem "-a"

# Nastaveni jmen souboru
RECORDER_FILE=Recorder

# Nastaveni cest
CONFIG_DIR=/mnt/sysdata/Config
CONFIG_BACKUP_DIR=/mnt/sysdata/Config.backup
PATH=/usr/bin:/bin:/usr/sbin:/sbin
LOGDIR=/mnt/sysdata/syslog/
LOGFILE=$LOGDIR/upgrade.log
FIFOFILE=/tmp/upgrade.fifo

# Nastaveni instalace
INSTALL_RECORDER=NO
ARCHIVE_CONFIG=NO
INSTALL_OK=NO	# Ukazuje celkovy stav instalace

# Konfigurace
TIMESTAMP_FORMAT="date +"%Y-%m-%d_%H:%M:%S""    # Podle date -I (ISO format)

trap "abort" EXIT INT TERM

# Funkce
print_help () {
echo -e >&2 \
"usage: $0 [-acr]\n" \
"       -a      Full installation\n" \
"	-c	Only copy configuration from update file\n" \
"	-r	Only copy Recorder binary"
}

cleanup () {
log "Firmware install - cleanup"
umount $REWRITE_RFS_MNT || log "Cannot umount RFS mountpoint: $REWRITE_RFS_MNT"
rmdir $REWRITE_RFS_MNT || log "Cannot remove RFS mount dir: $REWRITE_RFS_MNT"
rm -r $WORKDIR/* || log "Cannot erase workdir contents: $WORKDIR"
}

# Zapsani do log souboru
log () {
	TIMESTAMP="[ $(eval $TIMESTAMP_FORMAT) ]" # Ziskavam aktualni cas

        if [ -z "$1" ]; then
                echo $TIMESTAMP "Nothing to log.."  # no parameter passed.
        else
                echo $TIMESTAMP $1 >> $LOGFILE; echo $1
        fi
}

# Poslani zpravy do fifa
report () {
        if [ -z "$1" ]; then
                log "Warning: empty report message"
        else
                echo $1 > $FIFOFILE &
        fi
}

# Ukonceni skriptu
abort () {
	if [ "$INSTALL_OK" = "YES" ]; then
		log "Firmware install script exit OK"; report "349 100% 1"
	else
		log "Error 399: Firmware install script exit/abort"; report "-399"
	fi
	cleanup
}

# Funguju v adresari package/files
cd ..
WORKDIR=$PWD    # Jak moc je tohle bezpecne?

# Kontrola slozek
if [ ! -d "$LOGDIR" ]; then
	log "Creating log directory $LOGDIR"
	mkdir "$LOGDIR"
fi

# Zacatek instalace
log "Firmware installation started"; report "301 1% 1"

# Parsovani vstupnich parametru
while getopts ":acr:" opt;
do
	case "$opt" in
	# "-a" Full installation (default)
	a)	log "Config: full installation"
		INSTALL_RECORDER=YES
		ARCHIVE_CONFIG=YES
		;;
	# "-c" Only copy configuration from update package
	c)	log "Config: archive configuration only"
		ARCHIVE_CONFIG=YES
		;;
	# "-r" Only copy the Recorder binary
	r)	log "Config: copy Recorder binary only"
		INSTALL_RECORDER=YES
		;;
	# Unknown option. Print help
	\?)	print_help
		log "Error 351: Unknown argument"; report "-351"
		exit 1
		;;
	esac
done

# Zadny parametr - vypise napovedu
if [ "$INSTALL_RECORDER" != "YES" ] && [ "ARCHIVE_CONFIG" != "YES" ]; then
        print_help
	log "Error 351: no arguments specified"; report "-351"
        exit 1
fi

# Ktera kopie systemu prave bezi?
RFS0_TEST=`cat /proc/cmdline | grep -c ubi0:rootfs0`
RFS1_TEST=`cat /proc/cmdline | grep -c ubi0:rootfs1`

if [ "$RFS0_TEST" = "1" ] && [ "$RFS1_TEST" = "0" ]; then
        ACTIVE_SYS=0
        log "Active sys copy: 0"
elif [ "$RFS0_TEST" = "0" ] && [ "$RFS1_TEST" = 1 ]; then
        ACTIVE_SYS=1
        log "Active sys copy: 1"
else
        log "Error 352: determining active sys copy failed!"; report "-352"
        exit 1
fi

# Vyber prepisovaneho firmware, nastaveni cest
if [ "$ACTIVE_SYS" = "0" ]; then
        REWRITE_SYS=1
elif [ "$ACTIVE_SYS" = "1" ]; then
        REWRITE_SYS=0
else
        log "Error: Error!"
        exit 1
fi


# Mountnuti neaktivniho RFS
log "Mounting inactive RFS"; report "302 10% 1"
REWRITE_RFS_MNT=/mnt/rootfs"$REWRITE_SYS"
if [ ! -d "$REWRITE_RFS_MNT" ]; then
	mkdir "$REWRITE_RFS_MNT"
	log "Creating RFS mountpoint: $REWRITE_RFS_MNT"
fi
if ! df | grep -q /dev/ubi0_$REWRITE_SYS; then
if ! mount -t ubifs /dev/ubi0_"$REWRITE_SYS" "$REWRITE_RFS_MNT"; then
		log "Error 353: RFS mount failed!"; report "-353"
		exit 1
	fi
fi

# Skok do pracovniho adresare
cd $WORKDIR/files

# Kopirovani nove binarky Recorderu
log "Installing Recorder executable"; report "303 25% 1"
if [ "$INSTALL_RECORDER" = "YES" ]; then
	if ! cp $RECORDER_FILE $REWRITE_RFS_MNT/opt/; then
		log "Error 354: cannot copy Recorder binary!"; report "-354"
		exit 1
	fi
fi

# Archivace soucasneho nastaveni firmware

cd /mnt/sysdata

if [ "$ARCHIVE_CONFIG" = "YES" ]; then
	if [ ! -d $CONFIG_BACKUP_DIR ]; then
		mkdir "$CONFIG_BACKUP_DIR"
	fi
        FW_VERSION=`eval /opt/Recorder --version`
        SHORTTIME=`eval date +"%Y%m%d-%H%M%S"`
        # TODO: vycist odnekud ID modelu (uchovavano v EEPROM)
        MODEL_ID="1234567890"
	CONFIG_BACKUP_FILE=omr700-cfg-$FW_VERSION-$MODEL_ID-$SHORTTIME.tar.gz

	log "Archiving old configuration: $CONFIG_BACKUP_FILE"
	# Zabalim starou konfiguraci
	if ! tar -cvzf $CONFIG_BACKUP_DIR/$CONFIG_BACKUP_FILE -C $CONFIG_DIR .; then
		log "Error 357: cannot create backup configuration archive!"; report "-357"
		exit 1
	fi
fi

# Vse v poradku
INSTALL_OK=YES

# Konec
exit 0

